import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

enum CheckboxPosition { leading, trailing }

class CheckboxFormField extends FormField<bool> {
  CheckboxFormField(
      {Key key,
      String labelText,
      bool initialValue,
      FocusNode focusNode,
      bool readOnly = false,
      bool enabled = true,
      bool tristate = false,
      FormFieldSetter<bool> onSaved,
      FormFieldValidator<bool> validator,
      ValueChanged<bool> onChanged,
      CheckboxPosition checkboxPosition = CheckboxPosition.trailing,
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 4)})
      : super(
          key: key,
          initialValue:
              tristate == true ? initialValue : (initialValue ?? false),
          onSaved: onSaved,
          validator: validator,
          enabled: enabled,
          // ignore: missing_return
          builder: (field) {
            final theme = Theme.of(field.context);
            final label = Expanded(
                child: Text(
              labelText ?? '',
              style: enabled
                  ? null
                  : theme.textTheme.subtitle1
                      .copyWith(color: theme.disabledColor),
            ));
            var checkbox = Checkbox(
                focusNode: focusNode,
                tristate: tristate,
                onChanged: enabled == true
                    ? (value) {
                        if (readOnly == true) return;
                        field.didChange(value);
                        if (onChanged != null) onChanged(value);
                      }
                    : null,
                value: field.value,
                materialTapTargetSize: MaterialTapTargetSize.shrinkWrap);
            var children = <Widget>[label];
            if (checkboxPosition == CheckboxPosition.leading) {
              children.insert(0, checkbox);
            } else {
              children.add(checkbox);
            }
            return InkWell(
                onTap: () {
                  if (enabled == true && readOnly == false) {
                    if (tristate == true) {
                      if (field.value == true) {
                        field.didChange(null);
                      } else if (field.value == null) {
                        field.didChange(false);
                      } else {
                        field.didChange(true);
                      }
                    } else {
                      field.didChange(!field.value);
                    }
                  }
                },
                child: Padding(
                    padding: margin,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: children,
                    )));
          },
        );
}
