import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

enum RadioPosition { leading, trailing }

class RadioListFormField<T> extends FormField<T> {
  RadioListFormField(
      {Key key,
      String labelText,
      T initialValue,
      FocusNode focusNode,
      bool readOnly = false,
      bool enabled = true,
      Map<String, T> itemSource,
      FormFieldSetter<T> onSaved,
      FormFieldValidator<T> validator,
      ValueChanged<T> onChanged,
      double spacing = 4,
      Axis direction = Axis.horizontal,
      RadioPosition radioPosition = RadioPosition.trailing,
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 4)})
      : super(
          key: key,
          initialValue: initialValue,
          onSaved: onSaved,
          validator: validator,
          enabled: enabled,
          builder: (field) {
            var children = <Widget>[];
            itemSource.forEach((key, value) {
              children.add(RadioItem<T>(
                labelText: key,
                value: value,
                readOnly: readOnly,
                enabled: enabled,
                onChanged: (v) {
                  if (onChanged != null) onChanged(v);
                  field.didChange(v);
                },
                groupValue: field.value,
                radioPosition: radioPosition,
              ));
            });
            return Padding(
                padding: margin,
                child: Wrap(
                  spacing: spacing,
                  direction: direction,
                  children: children,
                ));
          },
        );
}

class RadioItem<T> extends StatelessWidget {
  final String labelText;
  final T value;
  final T groupValue;
  final FocusNode focusNode;
  final bool readOnly;

  final bool enabled;
  final ValueChanged<T> onChanged;
  final RadioPosition radioPosition;

  RadioItem(
      {Key key,
      this.labelText,
      this.value,
      this.readOnly = false,
      this.enabled = true,
      this.focusNode,
      this.onChanged,
      this.groupValue,
      this.radioPosition = RadioPosition.leading})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final theme = Theme.of(context);
    final label = Text(labelText ?? '',
        style: enabled
            ? null
            : theme.textTheme.subtitle1.copyWith(color: theme.disabledColor));
    var radio = Radio(
        focusNode: focusNode,
        groupValue: groupValue,
        onChanged: enabled == true
            ? (value) {
                if (readOnly == true) return;
                if (onChanged != null) onChanged(value);
              }
            : null,
        value: value,
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap);
    var children = <Widget>[label];
    if (radioPosition == RadioPosition.leading) {
      children.insert(0, radio);
    } else {
      children.add(radio);
    }
    return InkWell(
        onTap: () {
          if (enabled == true && readOnly == false) {
            if (onChanged != null) onChanged(value);
          }
        },
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: children,
        ));
  }
}
