import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

enum SwitchPosition { leading, trailing }

class SwitchFormField extends FormField<bool> {
  SwitchFormField(
      {Key key,
      String labelText,
      bool initialValue,
      FocusNode focusNode,
      bool readOnly = false,
      bool enabled = true,
      FormFieldSetter<bool> onSaved,
      FormFieldValidator<bool> validator,
      ValueChanged<bool> onChanged,
      SwitchPosition switchPosition = SwitchPosition.trailing,
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 4)})
      : super(
          key: key,
          initialValue: initialValue ?? false,
          onSaved: onSaved,
          validator: validator,
          enabled: enabled,
          // ignore: missing_return
          builder: (field) {
            final theme = Theme.of(field.context);
            final label = Expanded(
                child: Text(
              labelText ?? '',
              style: enabled
                  ? null
                  : theme.textTheme.subtitle1
                      .copyWith(color: theme.disabledColor),
            ));
            var switchButton = Switch(
                focusNode: focusNode,
                onChanged: enabled == true
                    ? (value) {
                        if (readOnly == true) return;
                        field.didChange(value);
                        if (onChanged != null) onChanged(value);
                      }
                    : null,
                value: field.value,
                materialTapTargetSize: MaterialTapTargetSize.shrinkWrap);
            var children = <Widget>[label];
            if (switchPosition == SwitchPosition.leading) {
              children.insert(0, switchButton);
            } else {
              children.add(switchButton);
            }
            return InkWell(
                onTap: () {
                  if (enabled == true && readOnly == false) {
                    field.didChange(!field.value);
                  }
                },
                child: Padding(
                    padding: margin,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: children,
                    )));
          },
        );
}
